@@ -4,6 +4,7 @@
 
 from typing import Any, Callable, Dict, List, Tuple, Union
 from operator import attrgetter
+import urllib.parse
 
 import numpy as np
 from sqlalchemy import Column, Boolean, Integer, String, Float, ForeignKey
@@ -60,11 +61,11 @@ class Element(Base):
         block (str): Block in periodic table, s, p, d, f
         boiling_point (float): Boiling temperature in K
         c6 (float): C_6 dispersion coefficient in a.u. from X. Chu & A. Dalgarno, J. Chem. Phys.,
-            121(9), 4083–4088 (2004) doi:10.1063/1.1779576, and the value for
+            121(9), 4083-4088 (2004) doi:10.1063/1.1779576, and the value for
             Hydrogen was taken from K. T. Tang, J. M. Norbeck and P. R. Certain,
             J. Chem. Phys. 64, 3063 (1976), doi:10.1063/1.432569
         c6_gb (float): C_6 dispersion coefficient in a.u. from Gould, T., & Bučko, T. (2016).
-            JCTC, 12(8), 3603–3613. http://doi.org/10.1021/acs.jctc.6b00361
+            JCTC, 12(8), 3603-3613. http://doi.org/10.1021/acs.jctc.6b00361
         cas (str): Chemical Abstracts Service identifier
         covalent_radius_bragg (float): Covalent radius in pm from
         covalent_radius_cordero (float): Covalent radius in pm from Cordero, B., Gómez, V., Platero-Prats, A.
@@ -73,7 +74,7 @@ class Element(Base):
             doi:10.1039/b801115j
         covalent_radius_pyykko (float): Single bond covalent radius in pm Pyykkö, P., & Atsumi, M. (2009).
             Molecular Single-Bond Covalent Radii for Elements 1-118.
-            Chemistry - A European Journal, 15(1), 186–197.
+            Chemistry - A European Journal, 15(1), 186-197.
             doi:10.1002/chem.200800987
         covalent_radius_pyykko_double (float): Double bond covalent radius in pm from P. Pyykkö et al.
         covalent_radius_pyykko_triple (float): Triple bond covalent radius in pm from P. Pyykkö et al.
@@ -96,6 +97,7 @@ class Element(Base):
         goldschmidt_class (str): Goldschmidt classification of the elements
         group_id (int): Group number
         heat_of_formation (float): Heat of formation in kJ/mol
+        inchi (str): International Chemical Identifier
         is_monoisotopic (bool): A flag marking if the element is monoisotopic
         jmol_color (str): Color of the atom as used in Jmol, in HEX,
             see http://jmol.sourceforge.net/jscolors/#color_U
@@ -110,15 +112,17 @@ class Element(Base):
             equation relating such distances with bond number
         metallic_radius_c12 (float): Metallic radius obtained by Pauling with an assumed number of
             nearest neighbors equal to 12
+        molar_heat_capacity (flaot): Molar heat capacity in J/mol K
         molcas_gv_color (str): Color of an atom in HEX from MOLCAS GV http://www.molcas.org/GV/
         name (str): Name in English
         name_origin (str): Origin of the name
+        nist_webbook_url (str): URL for the NIST Chemistry WebBook
         period (int): Period in periodic table
         pettifor_number (int): Pettifor scale
-        proton_affinity (Float): Proton affinity
+        proton_affinity (float): Proton affinity
         series (int): Index to chemical series
         sources (str): Sources of the element
-        specific_heat (float): Specific heat in J/g mol @ 20 C
+        specific_heat_capacity (float): Specific heat in J/g K @ 20 C
         symbol (str): Chemical symbol
         thermal_conductivity (float): Thermal conductivity in @/m K @25 C
         uses (str): Uses of the element
@@ -189,6 +193,7 @@ class Element(Base):
     mendeleev_number = Column(Integer)
     metallic_radius = Column(Float)
     metallic_radius_c12 = Column(Float)
+    molar_heat_capacity = Column(Float)
     molcas_gv_color = Column(String)
     name = Column(String)
     name_origin = Column(String)
@@ -199,7 +204,7 @@ class Element(Base):
     _series = relationship("Series", uselist=False, lazy="subquery")
     series = association_proxy("_series", "name")
     sources = Column(String)
-    specific_heat = Column(Float)
+    specific_heat_capacity = Column(Float)
     symbol = Column(String)
     thermal_conductivity = Column(Float)
     uses = Column(String)
@@ -225,20 +230,23 @@ class Element(Base):
 
         self.ec = ElectronicConfiguration(self.econf)
 
+    @hybrid_property
+    def specific_heat(self) -> float:
+        """Alias for `specific_heat_capacity` for backwards compatibility"""
+        return self.specific_heat_capacity
+
     @hybrid_property
     def ionenergies(self) -> Dict[int, float]:
         """
         Return a dict with ionization degree as keys and ionization energies
         in eV as values.
         """
-
         return {ie.degree: ie.energy for ie in self._ionization_energies}
 
     @hybrid_property
     def oxistates(self) -> List[int]:
-        """Return the oxidation states as a list of ints"""
-
-        return [os.oxidation_state for os in self._oxidation_states]
+        """Return the main oxidation states as a list of integers"""
+        return self.oxidation_states()
 
     @hybrid_property
     def sconst(self) -> Dict[Tuple[int, int], float]:
@@ -248,6 +256,20 @@ class Element(Base):
 
         return {(x.n, x.s): x.screening for x in self.screening_constants}
 
+    @hybrid_property
+    def inchi(self) -> str:
+        """International Chemical Identifier.
+
+        See: https://en.wikipedia.org/wiki/International_Chemical_Identifier
+        """
+        return f"InchI=1S/{self.symbol}"
+
+    @property
+    def nist_webbook_url(self) -> str:
+        """URL for the NIST Chemistry WebBook"""
+        nist_root_url = "https://webbook.nist.gov/cgi/inchi/"
+        return nist_root_url + urllib.parse.quote(self.inchi)
+
     @hybrid_property
     def electrons(self) -> int:
         """Return the number of electrons."""
@@ -320,7 +342,7 @@ class Element(Base):
 
     @hybrid_method
     def hardness(self, charge: int = 0) -> Union[float, None]:
-        """
+        r"""
         Return the absolute hardness, calculated as
 
         Args:
@@ -328,7 +350,7 @@ class Element(Base):
 
         .. math::
 
-           \eta = \\frac{IE - EA}{2}
+           \eta = \frac{IE - EA}{2}
 
         where:
 
@@ -354,13 +376,11 @@ class Element(Base):
             else:
                 return None
         elif charge < 0:
-            raise ValueError(
-                "Charge has to be a non-negative integer, got: {}".format(charge)
-            )
+            raise ValueError(f"Charge has to be a non-negative integer, got: {charge}")
 
     @hybrid_method
     def softness(self, charge: int = 0) -> Union[float, None]:
-        """
+        r"""
         Return the absolute softness, calculated as
 
         Args:
@@ -368,7 +388,7 @@ class Element(Base):
 
         .. math::
 
-           S = \\frac{1}{2\eta}
+           S = \frac{1}{2\eta}
 
         where :
 
@@ -378,10 +398,31 @@ class Element(Base):
 
         eta = self.hardness(charge=charge)
 
-        if eta is None:
-            return None
+        return None if eta is None else 1.0 / (2.0 * eta)
+
+    def oxidation_states(self, category: str = "main") -> List[int]:
+        """
+        Utility method for accessing oxidation states.
+
+        Args:
+            category (str): Category of oxidation state, Either
+                - `main` - for main, most common, oxidataion states
+                - `extended` - for less common oxidation states
+                - `all` - all oxidation states
+        """
+        if category not in {"main", "extended", "all"}:
+            raise ValueError(
+                f"got {category}, but allowed values are: 'main', 'extended', 'all'"
+            )
+
+        if category == "all":
+            return [o.oxidation_state for o in self._oxidation_states]
         else:
-            return 1.0 / (2.0 * eta)
+            return [
+                o.oxidation_state
+                for o in self._oxidation_states
+                if o.category == category
+            ]
 
     def zeff(
         self, n: int = None, o: str = None, method: str = "slater", alle: bool = False
@@ -417,13 +458,13 @@ class Element(Base):
         if n is None:
             n = self.ec.max_n()
         elif not isinstance(n, int):
-            raise ValueError("<n> should be an integer, got: {}".format(type(n)))
+            raise ValueError(f"<n> should be an integer, got: {type(n)}")
 
         if o is None:
             # take the shell with max `l` for a given `n`
             o = ORBITALS[max(get_l(x[1]) for x in self.ec.conf.keys() if x[0] == n)]
         elif o not in ORBITALS:
-            raise ValueError("<s> should be one of {}".format(", ".join(ORBITALS)))
+            raise ValueError(f'<s> should be one of {", ".join(ORBITALS)}')
 
         if method.lower() == "slater":
             return self.atomic_number - self.ec.slater_screening(n=n, o=o, alle=alle)
@@ -434,15 +475,15 @@ class Element(Base):
             else:
                 return sc
         else:
-            raise ValueError("<method> should be one of {}".format("slater, clementi"))
+            raise ValueError('<method> should be one of: "slater", "clementi"')
 
     def electrophilicity(self) -> Union[float, None]:
-        """
+        r"""
         Calculate electrophilicity index
 
         .. math::
 
-           \\omega = \\frac{\\mu}{2\eta}
+           \omega = \frac{\mu}{2\eta}
         """
 
         ip = self.ionenergies.get(1, None)
@@ -454,6 +495,7 @@ class Element(Base):
             return None
 
     def electronegativity_scales(self, name: str = None) -> Union[Callable, List[str]]:
+        # sourcery skip: assign-if-exp
         "Available electronegativity scales"
 
         scales = {
@@ -533,9 +575,7 @@ class Element(Base):
         """
 
         if (not isinstance(charge, int)) or (charge == 0):
-            raise ValueError(
-                "charge should be a nonzero initeger, got: {}".format(charge)
-            )
+            raise ValueError(f"charge should be a nonzero initeger, got: {charge}")
 
         if radius not in ["ionic_radius", "crystal_radius"]:
             raise ValueError(
@@ -556,14 +596,14 @@ class Element(Base):
         }
 
     def electronegativity_martynov_batsanov(self) -> float:
-        """
+        r"""
         Calculates the electronegativity value according to Martynov and
         Batsanov as the average of the ionization energies of the valence
         electrons
 
         .. math::
 
-           \chi_{MB} = \sqrt{\\frac{1}{n_{v}}\sum^{n_{v}}_{k=1} I_{k}}
+           \chi_{MB} = \sqrt{\frac{1}{n_{v}}\sum^{n_{v}}_{k=1} I_{k}}
 
         where:
 
@@ -588,7 +628,7 @@ class Element(Base):
         missing_is_zero: bool = False,
         allow_negative_ea: bool = False,
     ) -> float:
-        """
+        r"""
         Return the absolute electronegativity (Mulliken scale), calculated as
 
         Args:
@@ -598,7 +638,7 @@ class Element(Base):
 
         .. math::
 
-           \chi = \\frac{I + A}{2}
+           \chi = \frac{I + A}{2}
 
         where :math:`I` is the ionization energy and :math:`A` is the electron
         affinity
@@ -611,9 +651,7 @@ class Element(Base):
             ip = self.ionenergies.get(charge + 1, None)
             ea = self.ionenergies.get(charge, None)
         else:
-            raise ValueError(
-                "Charge has to be a non-negative integer, got: {}".format(charge)
-            )
+            raise ValueError(f"Charge has to be a non-negative integer, got: {charge}")
         return mulliken(
             ip, ea, missing_is_zero=missing_is_zero, allow_negative_ea=allow_negative_ea
         )
@@ -648,7 +686,7 @@ class Element(Base):
 
     def oxides(self) -> List[str]:
         """
-        Return a lsit of possible oxides based on the oxidation number
+        Return a list of possible oxides based on the oxidation number
         """
 
         oxide_coeffs = [coeffs(ox) for ox in self.oxistates if ox > 0]
@@ -656,6 +694,31 @@ class Element(Base):
         normal_coeffs = [[str(c) if c != 1 else "" for c in t] for t in oxide_coeffs]
         return [f"{self.symbol}{cme}O{co}" for cme, co in normal_coeffs]
 
+    def __hash__(self):
+        """Custom has function to allow comparisons
+
+        This drops allt he nested, related objects since SQLAlchemy use a custom
+        unhashable `InstrumentedList`.
+        """
+        to_drop = [
+            "_ionization_energies",
+            "_oxidation_states",
+            "_sa_instance_state",
+            "_series",
+            "_series_id",
+            "ec",
+            "group",
+            "ionic_radii",
+            "isotopes",
+            "screening_constants",
+        ]
+        hashable = [(k, v) for k, v in self.__dict__.items() if k not in to_drop]
+        return hash(tuple(sorted(hashable)))
+
+    def __eq__(self, other):
+        """Overwrite the defalt comparison"""
+        return hash(self) == hash(other)
+
     def __str__(self):
         return "{0} {1} {2}".format(self.atomic_number, self.symbol, self.name)
 
@@ -817,6 +880,8 @@ class OxidationState(Base):
     Args:
       atomic_number (int): Atomic number
       oxidation_state (int): Oxidation state
+      category (str): Either `main` or `extended` flag to indicate
+        the type of oxidation state.
     """
 
     __tablename__ = "oxidationstates"
@@ -824,11 +889,16 @@ class OxidationState(Base):
     id = Column(Integer, primary_key=True)
     atomic_number = Column(Integer, ForeignKey("elements.atomic_number"))
     oxidation_state = Column(Integer)
+    category = Column(String)
 
     def __repr__(self):
-
-        return "<OxidationState(atomic_number={a:5d}, oxidation_state={o:5d})>".format(
-            a=self.atomic_number, o=self.oxidation_state
+        return ", ".join(
+            [
+                f"<OxidationState(id={self.id}",
+                f"atomic_number={self.atomic_number}",
+                f"oxidation_state={self.oxidation_state}",
+                f"category={self.category})>",
+            ]
         )
 
 
@@ -881,41 +951,54 @@ class Isotope(Base):
 
     Args:
         abundance (float): Abundance of the isotope
+        abundance_uncertainty (float): Abundance uncertainty
         atomic_number (int): Atomic number
+        discovery_year (int): Year the isotope was discovered
+        g_factor (float): Dimensionless magnetic moment
+        g_factor_uncertainty (float): Uncertainty for the `g_factor`
         half_life (float): Half life time
+        half_life_uncertainty (float): Uncertainty for the `half_life`
         half_life_unit (str): Unit for the half life time
         is_radioactive (bool): A flag marking wheather the isotope is radioactive
         mass (float): Mass of the isotope
         mass_number (int): Mass number of the isotope
         mass_uncertainty (float): Uncertainty of the mass value
+        parity (str): Parity, if present, it can be either `+` or `-`
+        quadrupole_moment (float): Quadrupole moment
+        quadrupole_moment_uncertainty (float): Uncertainty for the `quadrupole_moment`
+        spin (str): Nuclear spin
     """
 
     __tablename__ = "isotopes"
 
     id = Column(Integer, primary_key=True)
     abundance = Column(Float)
+    abundance_uncertainty = Column(Float)
     atomic_number = Column(Integer, ForeignKey("elements.atomic_number"))
+    discovery_year = Column(Integer)
     g_factor = Column(Float)
+    g_factor_uncertainty = Column(Float)
     half_life = Column(Float)
+    half_life_uncertainty = Column(Float)
     half_life_unit = Column(String)
-    is_radioactive = Column(Boolean)
-    mass = Column(Float)
-    mass_number = Column(Integer)
-    mass_uncertainty = Column(Float)
-    spin = Column(Float)
+    is_radioactive = Column(Boolean, nullable=False)
+    mass = Column(Float, nullable=False)
+    mass_number = Column(Integer, nullable=False)
+    mass_uncertainty = Column(Float, nullable=False)
+    parity = Column(String)
     quadrupole_moment = Column(Float)
+    quadrupole_moment_uncertainty = Column(Float)
+    spin = Column(String)
+
+    @hybrid_property
+    def is_stable(self) -> bool:
+        """Flag to indicate whether the isotope is stable"""
+        return not self.is_radioactive
 
     def __str__(self):
 
-        afmt = "5.3f"
-        mfmt = "10.5f"
-
-        if self.mass is None:
-            mfmt = ""
-
-        if self.abundance is None:
-            afmt = ""
-
+        mfmt = "" if self.mass is None else "10.5f"
+        afmt = "" if self.abundance is None else "5.3f"
         return "{0:5d} {1:5d} {2:{mfmt}} {3:{afmt}}".format(
             self.atomic_number,
             self.mass_number,
@@ -926,10 +1009,7 @@ class Isotope(Base):
         )
 
     def __repr__(self):
-
-        return "<Isotope(Z={}, A={}, mass={}, abundance={})>".format(
-            self.atomic_number, self.mass_number, self.mass, self.abundance
-        )
+        return f"<Isotope(Z={self.atomic_number}, A={self.mass_number}, mass={self.mass}, abundance={self.abundance})>"
 
 
 class ScreeningConstant(Base):
